@@ -274,7 +274,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         )
 
         toggleactions = (
-            ("IgnoreCase",   gtk.STOCK_ITALIC,  _("Ignore filename case"),     None, _("Consider differently-cased filenames that are otherwise-identical to be the same"), self.on_button_ignore_case_toggled, False),
+            ("IgnoreCase",   gtk.STOCK_ITALIC,  _("Ignore Filename Case"), None, _("Consider differently-cased filenames that are otherwise-identical to be the same"), self.on_button_ignore_case_toggled, False),
             ("ShowSame",     gtk.STOCK_APPLY,   _("Same"),     None, _("Show identical"), self.on_filter_state_toggled, False),
             ("ShowNew",      gtk.STOCK_ADD,     _("New"),      None, _("Show new"), self.on_filter_state_toggled, False),
             ("ShowModified", gtk.STOCK_REMOVE,  _("Modified"), None, _("Show modified"), self.on_filter_state_toggled, False),
@@ -340,6 +340,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             col_index = self.model.column_index
             # Create icon and filename CellRenderer
             column = gtk.TreeViewColumn(_("Name"))
+            column.set_resizable(True)
             rentext = gtk.CellRendererText()
             renicon = emblemcellrenderer.EmblemCellRenderer()
             column.pack_start(renicon, expand=0)
@@ -357,6 +358,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             self.columns_dict[i]["name"] = column
             # Create file size CellRenderer
             column = gtk.TreeViewColumn(_("Size"))
+            column.set_resizable(True)
             rentext = gtk.CellRendererText()
             column.pack_start(rentext, expand=1)
             column.set_attributes(rentext, markup=col_index(COL_SIZE, i))
@@ -364,6 +366,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             self.columns_dict[i]["size"] = column
             # Create date-time CellRenderer
             column = gtk.TreeViewColumn(_("Modification time"))
+            column.set_resizable(True)
             rentext = gtk.CellRendererText()
             column.pack_start(rentext, expand=1)
             column.set_attributes(rentext, markup=col_index(COL_TIME, i))
@@ -371,6 +374,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             self.columns_dict[i]["modification time"] = column
             # Create permissions CellRenderer
             column = gtk.TreeViewColumn(_("Permissions"))
+            column.set_resizable(True)
             rentext = gtk.CellRendererText()
             column.pack_start(rentext, expand=0)
             column.set_attributes(rentext, markup=col_index(COL_PERMS, i))
@@ -586,6 +590,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         # This is difficult to trigger, and to test. Most of the time here we
         # will actually have had UTF-8 from GTK, which has been unicode-ed by
         # the time we get this far. This is a fallback, and may be wrong!
+        locations = list(locations)
         for i, l in enumerate(locations):
             if not isinstance(l, unicode):
                 locations[i] = l.decode(sys.getfilesystemencoding())
@@ -1448,7 +1453,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             self.treeview[pane].expand_to_path(path)
             self.treeview[pane].set_cursor(path)
 
-    def on_reload_activate(self, *extra):
+    def on_refresh_activate(self, *extra):
         self.on_fileentry_activate(None)
 
     def on_delete_event(self, appquit=0):
