@@ -146,7 +146,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                  gtk.keysyms.Control_R : MASK_CTRL}
 
     # Identifiers for MsgArea messages
-    (MSG_SAME,) = list(range(1))
+    (MSG_SAME, MSG_SLOW_HIGHLIGHT) = list(range(2))
 
     __gsignals__ = {
         'next-conflict-changed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (bool, bool)),
@@ -208,6 +208,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self._scroll_lock = False
         self.linediffer = self.differ()
         self.linediffer.ignore_blanks = self.prefs.ignore_blank_lines
+        self.force_highlight = False
         self.syncpoints = []
         self.in_nested_textview_gutter_expose = False
         self._cached_match = CachedSequenceMatcher()
@@ -217,34 +218,78 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             buf.create_tag("inline")
 
         actions = (
-            ("MakePatch", None, _("Format as patch..."), None, _("Create a patch using differences between files"), self.make_patch),
-            ("SplitAdd", None, _("Add synchronization point"), None,
+            ("MakePatch", None, _("Format as Patch..."), None,
+                _("Create a patch using differences between files"),
+                self.make_patch),
+            ("SaveAll", None, _("Save A_ll"), "<Ctrl><Shift>L",
+                _("Save all files in the current comparison"),
+                self.on_save_all_activate),
+            ("Revert", gtk.STOCK_REVERT_TO_SAVED, None, None,
+                _("Revert files to their saved versions"),
+                self.on_revert_activate),
+            ("SplitAdd", None, _("Add Synchronization Point"), None,
                 _("Add a manual point for synchronization of changes between "
                   "files"),
                 self.add_sync_point),
-            ("SplitClear", None, _("Clear synchronization points"), None,
+            ("SplitClear", None, _("Clear Synchronization Points"), None,
                 _("Clear manual change sychronization points"),
                 self.clear_sync_points),
-            ("PrevConflict", None, _("Previous conflict"), "<Ctrl>I", _("Go to the previous conflict"), lambda x: self.on_next_conflict(gtk.gdk.SCROLL_UP)),
-            ("NextConflict", None, _("Next conflict"), "<Ctrl>K", _("Go to the next conflict"), lambda x: self.on_next_conflict(gtk.gdk.SCROLL_DOWN)),
-            ("PushLeft",  gtk.STOCK_GO_BACK,    _("Push to left"),    "<Alt>Left", _("Push current change to the left"), lambda x: self.push_change(-1)),
-            ("PushRight", gtk.STOCK_GO_FORWARD, _("Push to right"),   "<Alt>Right", _("Push current change to the right"), lambda x: self.push_change(1)),
+            ("PrevConflict", None, _("Previous Conflict"), "<Ctrl>I",
+                _("Go to the previous conflict"),
+                lambda x: self.on_next_conflict(gtk.gdk.SCROLL_UP)),
+            ("NextConflict", None, _("Next Conflict"), "<Ctrl>K",
+                _("Go to the next conflict"),
+                lambda x: self.on_next_conflict(gtk.gdk.SCROLL_DOWN)),
+            ("PushLeft", gtk.STOCK_GO_BACK, _("Push to Left"), "<Alt>Left",
+                _("Push current change to the left"),
+                lambda x: self.push_change(-1)),
+            ("PushRight", gtk.STOCK_GO_FORWARD,
+                _("Push to Right"), "<Alt>Right",
+                _("Push current change to the right"),
+                lambda x: self.push_change(1)),
             # FIXME: using LAST and FIRST is terrible and unreliable icon abuse
-            ("PullLeft",  gtk.STOCK_GOTO_LAST,  _("Pull from left"),  "<Alt><Shift>Right", _("Pull change from the left"), lambda x: self.pull_change(-1)),
-            ("PullRight", gtk.STOCK_GOTO_FIRST, _("Pull from right"), "<Alt><Shift>Left", _("Pull change from the right"), lambda x: self.pull_change(1)),
-            ("CopyLeftUp", None, _("Copy above left"), "<Alt>bracketleft", _("Copy change above the left chunk"), lambda x: self.copy_change(-1, -1)),
-            ("CopyLeftDown", None, _("Copy below left"), "<Alt>semicolon", _("Copy change below the left chunk"), lambda x: self.copy_change(-1, 1)),
-            ("CopyRightUp", None, _("Copy above right"), "<Alt>bracketright", _("Copy change above the right chunk"), lambda x: self.copy_change(1, -1)),
-            ("CopyRightDown", None, _("Copy below right"), "<Alt>quoteright", _("Copy change below the right chunk"), lambda x: self.copy_change(1, 1)),
-            ("Delete",    gtk.STOCK_DELETE,     _("Delete"),     "<Alt>Delete", _("Delete change"), self.delete_change),
-            ("MergeFromLeft",  None, _("Merge all changes from left"),  None, _("Merge all non-conflicting changes from the left"), lambda x: self.pull_all_non_conflicting_changes(-1)),
-            ("MergeFromRight", None, _("Merge all changes from right"), None, _("Merge all non-conflicting changes from the right"), lambda x: self.pull_all_non_conflicting_changes(1)),
-            ("MergeAll",       None, _("Merge all non-conflicting"),    None, _("Merge all non-conflicting changes from left and right panes"), lambda x: self.merge_all_non_conflicting_changes()),
-            ("CycleDocuments", None, _("Cycle through documents"), "<control>Escape", _("Move keyboard focus to the next document in this comparison"), self.action_cycle_documents),
+            ("PullLeft", gtk.STOCK_GOTO_LAST,
+                _("Pull from Left"), "<Alt><Shift>Right",
+                _("Pull change from the left"),
+                lambda x: self.pull_change(-1)),
+            ("PullRight", gtk.STOCK_GOTO_FIRST,
+                _("Pull from Right"), "<Alt><Shift>Left",
+                _("Pull change from the right"),
+                lambda x: self.pull_change(1)),
+            ("CopyLeftUp", None, _("Copy Above Left"), "<Alt>bracketleft",
+                _("Copy change above the left chunk"),
+                lambda x: self.copy_change(-1, -1)),
+            ("CopyLeftDown", None, _("Copy Below Left"), "<Alt>semicolon",
+                _("Copy change below the left chunk"),
+                lambda x: self.copy_change(-1, 1)),
+            ("CopyRightUp", None, _("Copy Above Right"), "<Alt>bracketright",
+                _("Copy change above the right chunk"),
+                lambda x: self.copy_change(1, -1)),
+            ("CopyRightDown", None, _("Copy Below Right"), "<Alt>quoteright",
+                _("Copy change below the right chunk"),
+                lambda x: self.copy_change(1, 1)),
+            ("Delete", gtk.STOCK_DELETE, _("Delete"), "<Alt>Delete",
+                _("Delete change"),
+                self.delete_change),
+            ("MergeFromLeft", None, _("Merge All from Left"), None,
+                _("Merge all non-conflicting changes from the left"),
+                lambda x: self.pull_all_non_conflicting_changes(-1)),
+            ("MergeFromRight", None, _("Merge All from Right"), None,
+                _("Merge all non-conflicting changes from the right"),
+                lambda x: self.pull_all_non_conflicting_changes(1)),
+            ("MergeAll", None, _("Merge All"), None,
+                _("Merge all non-conflicting changes from left and right "
+                  "panes"),
+                lambda x: self.merge_all_non_conflicting_changes()),
+            ("CycleDocuments", None,
+                _("Cycle Through Documents"), "<control>Escape",
+                _("Move keyboard focus to the next document in this "
+                  "comparison"),
+                self.action_cycle_documents),
         )
 
         toggle_actions = (
-            ("LockScrolling", None, _("Lock scrolling"), None,
+            ("LockScrolling", None, _("Lock Scrolling"), None,
              _("Lock scrolling of all panes"),
              self.on_action_lock_scrolling_toggled, True),
         )
@@ -275,10 +320,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.connect("next-conflict-changed", self.on_next_conflict_changed)
 
         overwrite_label = gtk.Label()
-        overwrite_label.set_size_request(50, -1)
         overwrite_label.show()
         cursor_label = gtk.Label()
-        cursor_label.set_size_request(150, -1)
         cursor_label.show()
         self.status_info_labels = [overwrite_label, cursor_label]
 
@@ -314,7 +357,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                             "delete"  : lookup("insert-outline", "#77f077"),
                             "conflict": lookup("conflict-outline", "#f0768b"),
                             "replace" : lookup("replace-outline", "#8bbff3")}
-        self.highlight_color = lookup("highlight-bg", "#ffff00")
+        self.highlight_color = lookup("current-line-highlight", "#ffff00")
         self.syncpoint_color = lookup("syncpoint-outline", "#555555")
 
         for associated in self.diffmap + self.linkmap:
@@ -384,10 +427,6 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
 
         insert_overwrite = self._insert_overwrite_text[self.textview_overwrite]
         line_column = self._line_column_text % (line + 1, offset + 1)
-        status = "%s : %s" % (insert_overwrite, line_column)
-        # FIXME: Think this status-changed is wrong...
-        # self.emit("status-changed", status)
-
         self.status_info_labels[0].set_text(insert_overwrite)
         self.status_info_labels[1].set_text(line_column)
 
@@ -876,7 +915,10 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         pane = self._get_focused_pane()
         if pane >= 0:
             if self.textbuffer[pane].data.filename:
-                self._open_files([self.textbuffer[pane].data.filename])
+                pos = self.textbuffer[pane].props.cursor_position
+                cursor_it = self.textbuffer[pane].get_iter_at_offset(pos)
+                line = cursor_it.get_line() + 1
+                self._open_files([self.textbuffer[pane].data.filename], line)
 
     def get_selected_text(self):
         """Returns selected text of active pane"""
@@ -962,6 +1004,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         modified = False if pane == -1 else self.textbuffer[pane].data.modified
         if self.main_actiongroup:
             self.main_actiongroup.get_action("Save").set_sensitive(modified)
+        any_modified = any(b.data.modified for b in self.textbuffer)
+        self.actiongroup.get_action("SaveAll").set_sensitive(any_modified)
 
     def recompute_label(self):
         self._set_save_action_sensitivity()
@@ -992,6 +1036,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
            If an element is None, the text of a pane is left as is.
         """
         self._disconnect_buffer_handlers()
+        files = list(files)
         for i, f in enumerate(files):
             if not f:
                 continue
@@ -1101,20 +1146,28 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         for b in self.textbuffer:
             self.undosequence.checkpoint(b)
 
-    def _diff_files(self):
+    def _diff_files(self, refresh=False):
         yield _("[%s] Computing differences") % self.label_text
         texts = self.buffer_filtered[:self.num_panes]
         step = self.linediffer.set_sequences_iter(texts)
         while next(step) is None:
             yield 1
 
-        chunk, prev, next_ = self.linediffer.locate_chunk(1, 0)
-        self.cursor.next = chunk
-        if self.cursor.next is None:
-            self.cursor.next = next_
-        for buf in self.textbuffer:
-            buf.place_cursor(buf.get_start_iter())
-        self.scheduler.add_task(lambda: self.next_diff(gtk.gdk.SCROLL_DOWN), True)
+        if not refresh:
+            chunk, prev, next_ = self.linediffer.locate_chunk(1, 0)
+            self.cursor.next = chunk
+            if self.cursor.next is None:
+                self.cursor.next = next_
+            for buf in self.textbuffer:
+                buf.place_cursor(buf.get_start_iter())
+
+            if self.cursor.next is not None:
+                self.scheduler.add_task(
+                    lambda: self.next_diff(gtk.gdk.SCROLL_DOWN, True), True)
+            else:
+                buf = self.textbuffer[1 if self.num_panes > 1 else 0]
+                self.on_cursor_position_changed(buf, None, True)
+
         self.queue_draw()
         self._connect_buffer_handlers()
         self._set_merge_action_sensitivity()
@@ -1154,7 +1207,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             buf.remove_tag(tag, buf.get_start_iter(), buf.get_end_iter())
 
         self.queue_draw()
-        self.scheduler.add_task(self._diff_files())
+        self.scheduler.add_task(self._diff_files(refresh=True))
 
     def _set_merge_action_sensitivity(self):
         pane = self._get_focused_pane()
@@ -1190,8 +1243,10 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 bufs = self.textbuffer[1], self.textbuffer[to_idx]
                 tags = alltags[1], alltags[to_idx]
 
-                starts = [b.get_iter_at_line_or_eof(l) for b, l in zip(bufs, (c[1], c[3]))]
-                ends = [b.get_iter_at_line_or_eof(l) for b, l in zip(bufs, (c[2], c[4]))]
+                starts = [b.get_iter_at_line_or_eof(l) for b, l in
+                          zip(bufs, (c[1], c[3]))]
+                ends = [b.get_iter_at_line_or_eof(l) for b, l in
+                        zip(bufs, (c[2], c[4]))]
                 bufs[0].remove_tag(tags[0], starts[0], ends[0])
                 bufs[1].remove_tag(tags[1], starts[1], ends[1])
 
@@ -1204,8 +1259,10 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 bufs = self.textbuffer[1], self.textbuffer[to_idx]
                 tags = alltags[1], alltags[to_idx]
 
-                starts = [b.get_iter_at_line_or_eof(l) for b, l in zip(bufs, (c[1], c[3]))]
-                ends = [b.get_iter_at_line_or_eof(l) for b, l in zip(bufs, (c[2], c[4]))]
+                starts = [b.get_iter_at_line_or_eof(l) for b, l in
+                          zip(bufs, (c[1], c[3]))]
+                ends = [b.get_iter_at_line_or_eof(l) for b, l in
+                        zip(bufs, (c[2], c[4]))]
 
                 # We don't use self.buffer_texts here, as removing line
                 # breaks messes with inline highlighting in CRLF cases
@@ -1214,11 +1271,13 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 textn = bufs[1].get_text(starts[1], ends[1], False)
                 textn = text_type(textn, 'utf8')
 
-                # For very long sequences, bail rather than trying a very slow comparison
-                inline_limit = 8000
-                if len(text1) + len(textn) > inline_limit:
+                # Bail on long sequences, rather than try a slow comparison
+                inline_limit = 10000
+                if len(text1) + len(textn) > inline_limit and \
+                        not self.force_highlight:
                     for i in range(2):
                         bufs[i].apply_tag(tags[i], starts[i], ends[i])
+                    self._prompt_long_highlighting()
                     continue
 
                 def apply_highlight(bufs, tags, starts, ends, texts, matches):
@@ -1234,10 +1293,11 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                     if texts != (text1, textn):
                         return
 
-                    # Remove equal matches of size greater than 3; highlight
+                    # Remove equal matches of size less than 3; highlight
                     # the remainder.
                     matches = [m for m in matches if m.tag != "equal" or
-                        (m.end_a - m.start_a < 3) or (m.end_b - m.start_b < 3)]
+                               (m.end_a - m.start_a < 3) or
+                               (m.end_b - m.start_b < 3)]
 
                     for i in range(2):
                         start, end = starts[i].copy(), starts[i].copy()
@@ -1300,6 +1360,35 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 if m.get_msg_id() == FileDiff.MSG_SAME:
                     m.clear()
 
+    def _prompt_long_highlighting(self):
+
+        def on_msgarea_highlighting_response(msgarea, respid):
+            for mgr in self.msgarea_mgr:
+                mgr.clear()
+            if respid == gtk.RESPONSE_OK:
+                self.force_highlight = True
+                self.refresh_comparison()
+
+        for index, mgr in enumerate(self.msgarea_mgr):
+            msgarea = mgr.new_from_text_and_icon(
+                gtk.STOCK_INFO,
+                _("Change highlighting incomplete"),
+                _("Some changes were not highlighted because they were too "
+                  "large. You can force Meld to take longer to highlight "
+                  "larger changes, though this may be slow."))
+            mgr.set_msg_id(FileDiff.MSG_SLOW_HIGHLIGHT)
+            button = msgarea.add_stock_button_with_text(
+                _("Hide"), gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
+            if index == 0:
+                button.props.label = _("Hi_de")
+            button = msgarea.add_button(
+                _("Keep highlighting"), gtk.RESPONSE_OK)
+            if index == 0:
+                button.props.label = _("_Keep highlighting")
+            msgarea.connect("response",
+                            on_msgarea_highlighting_response)
+            msgarea.show_all()
+
     def on_msgarea_identical_response(self, msgarea, respid):
         for mgr in self.msgarea_mgr:
             mgr.clear()
@@ -1439,14 +1528,21 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             return False
         return True
 
-    def save_file(self, pane, saveas=0):
+    def save_file(self, pane, saveas=False):
         buf = self.textbuffer[pane]
         bufdata = buf.data
-        if saveas or not bufdata.filename or not bufdata.writable:
-            prompt = _("Choose a name for buffer %i.") % (pane + 1)
+        if saveas or not (bufdata.filename or bufdata.savefile) \
+                or not bufdata.writable:
+            if pane == 0:
+                prompt = _("Save Left Pane As")
+            elif pane == 1 and self.num_panes == 3:
+                prompt = _("Save Middle Pane As")
+            else:
+                prompt = _("Save Right Pane As")
             filename = self._get_filename_for_saving(prompt)
             if filename:
                 bufdata.filename = bufdata.label = os.path.abspath(filename)
+                bufdata.savefile = None
                 self.fileentry[pane].set_filename(bufdata.filename)
                 self.fileentry[pane].prepend_history(bufdata.filename)
             else:
@@ -1525,7 +1621,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         if pane >= 0:
             self.save_file(pane, True)
 
-    def save_all(self):
+    def on_save_all_activate(self, action):
         for i in range(self.num_panes):
             if self.textbuffer[i].data.modified:
                 self.save_file(i)
@@ -1544,9 +1640,21 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 return i
         return -1
 
-    def on_reload_activate(self, *extra):
-        primary = _("Save changes to documents before reloading?")
-        if self.check_save_modified(primary) != gtk.RESPONSE_CANCEL:
+    def on_revert_activate(self, *extra):
+        response = gtk.RESPONSE_OK
+        unsaved = [b.data.label for b in self.textbuffer if b.data.modified]
+        if unsaved:
+            ui_path = paths.ui_dir("filediff.ui")
+            dialog = gnomeglade.Component(ui_path, "revert_dialog")
+            dialog.widget.set_transient_for(self.widget.get_toplevel())
+            # FIXME: Should be packed into dialog.widget.get_message_area(),
+            # but this is unbound on currently required PyGTK.
+            filelist = "\n".join(["\t" + f for f in unsaved])
+            dialog.widget.props.secondary_text += filelist
+            response = dialog.widget.run()
+            dialog.widget.destroy()
+
+        if response == gtk.RESPONSE_OK:
             files = [b.data.filename for b in self.textbuffer[:self.num_panes]]
             self.set_files(files)
 
@@ -1705,7 +1813,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             self.queue_draw()
             self.recompute_label()
 
-    def next_diff(self, direction):
+    def next_diff(self, direction, centered=False):
         pane = self._get_focused_pane()
         if pane == -1:
             if len(self.textview) > 1:
@@ -1727,7 +1835,11 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             # Warp the cursor to the first line of next chunk
             if self.cursor.line != c[1]:
                 buf.place_cursor(buf.get_iter_at_line(c[1]))
-            self.textview[pane].scroll_to_mark(buf.get_insert(), 0.1)
+            if centered:
+                self.textview[pane].scroll_to_mark(buf.get_insert(), 0.0,
+                                                   True)
+            else:
+                self.textview[pane].scroll_to_mark(buf.get_insert(), 0.2)
 
     def copy_chunk(self, src, dst, chunk, copy_up):
         b0, b1 = self.textbuffer[src], self.textbuffer[dst]
