@@ -13,14 +13,15 @@
 
 ### You should have received a copy of the GNU General Public License
 ### along with this program; if not, write to the Free Software
-### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+### USA.
 
 
 import math
 
 import gtk
 
-import diffutil
+from . import diffutil
 
 
 # FIXME: import order issues
@@ -172,7 +173,7 @@ class LinkMap(gtk.DrawingArea):
         # For bezier control points
         x_steps = [-0.5, (1. / 3) * wtotal, (2. / 3) * wtotal, wtotal + 0.5]
         # Rounded rectangle corner radius for culled changes display
-        radius = self.line_height / 2
+        radius = self.line_height // 2
         q_rad = math.pi / 2
 
         left, right = self.view_indices
