@@ -38,7 +38,6 @@ from .ui import gnomeglade
 from .ui import notebooklabel
 
 from .util.compat import string_types
-from .util.sourceviewer import srcviewer
 from .meldapp import app
 
 
@@ -60,52 +59,110 @@ class MeldWindow(gnomeglade.Component):
 
         actions = (
             ("FileMenu", None, _("_File")),
-            ("New",     gtk.STOCK_NEW,      _("_New comparison"), "<control>N", _("Start a new comparison"), self.on_menu_file_new_activate),
-            ("Save",    gtk.STOCK_SAVE,     None, None, _("Save the current file"), self.on_menu_save_activate),
-            ("SaveAs",  gtk.STOCK_SAVE_AS,  None, "<control><shift>S", "Save the current file with a different name", self.on_menu_save_as_activate),
-            ("Close",   gtk.STOCK_CLOSE,    None, None, _("Close the current file"), self.on_menu_close_activate),
-            ("Quit",    gtk.STOCK_QUIT,     None, None, _("Quit the program"), self.on_menu_quit_activate),
+            ("New", gtk.STOCK_NEW, _("_New Comparison..."), "<control>N",
+                _("Start a new comparison"),
+                self.on_menu_file_new_activate),
+            ("Save", gtk.STOCK_SAVE, None, None,
+                _("Save the current file"),
+                self.on_menu_save_activate),
+            ("SaveAs", gtk.STOCK_SAVE_AS, _("Save As..."), "<control><shift>S",
+                _("Save the current file with a different name"),
+                self.on_menu_save_as_activate),
+            ("Close", gtk.STOCK_CLOSE, None, None,
+                _("Close the current file"),
+                self.on_menu_close_activate),
+            ("Quit", gtk.STOCK_QUIT, None, None,
+                _("Quit the program"),
+                self.on_menu_quit_activate),
 
             ("EditMenu", None, _("_Edit")),
-            ("Undo",    gtk.STOCK_UNDO,     None, "<control>Z", _("Undo the last action"), self.on_menu_undo_activate),
-            ("Redo",    gtk.STOCK_REDO,     None, "<control><shift>Z", _("Redo the last undone action"), self.on_menu_redo_activate),
-            ("Cut",     gtk.STOCK_CUT,      None, None, _("Cut the selection"), self.on_menu_cut_activate),
-            ("Copy",    gtk.STOCK_COPY,     None, None, _("Copy the selection"), self.on_menu_copy_activate),
-            ("Paste",   gtk.STOCK_PASTE,    None, None, _("Paste the clipboard"), self.on_menu_paste_activate),
-            ("Find",    gtk.STOCK_FIND,     None, None, _("Search for text"), self.on_menu_find_activate),
-            ("FindNext", None,              _("Find Ne_xt"), "<control>G", _("Search forwards for the same text"), self.on_menu_find_next_activate),
-            ("FindPrevious", None,          _("Find _Previous"), "<control><shift>G", _("Search backwards for the same text"), self.on_menu_find_previous_activate),
-            ("Replace", gtk.STOCK_FIND_AND_REPLACE, _("_Replace"), "<control>H", _("Find and replace text"), self.on_menu_replace_activate),
-            ("Preferences", gtk.STOCK_PREFERENCES, _("Prefere_nces"), None, _("Configure the application"), self.on_menu_preferences_activate),
+            ("Undo", gtk.STOCK_UNDO, None, "<control>Z",
+                _("Undo the last action"),
+                self.on_menu_undo_activate),
+            ("Redo", gtk.STOCK_REDO, None, "<control><shift>Z",
+                _("Redo the last undone action"),
+                self.on_menu_redo_activate),
+            ("Cut", gtk.STOCK_CUT, None, None, _("Cut the selection"),
+                self.on_menu_cut_activate),
+            ("Copy", gtk.STOCK_COPY, None, None, _("Copy the selection"),
+                self.on_menu_copy_activate),
+            ("Paste", gtk.STOCK_PASTE, None, None, _("Paste the clipboard"),
+                self.on_menu_paste_activate),
+            ("Find", gtk.STOCK_FIND, _("Find..."), None, _("Search for text"),
+                self.on_menu_find_activate),
+            ("FindNext", None, _("Find Ne_xt"), "<control>G",
+                _("Search forwards for the same text"),
+                self.on_menu_find_next_activate),
+            ("FindPrevious", None, _("Find _Previous"), "<control><shift>G",
+                _("Search backwards for the same text"),
+                self.on_menu_find_previous_activate),
+            ("Replace", gtk.STOCK_FIND_AND_REPLACE,
+                _("_Replace..."), "<control>H",
+                _("Find and replace text"),
+                self.on_menu_replace_activate),
+            ("Preferences", gtk.STOCK_PREFERENCES, _("Prefere_nces"), None,
+                _("Configure the application"),
+                self.on_menu_preferences_activate),
 
             ("ChangesMenu", None, _("_Changes")),
-            ("NextChange", gtk.STOCK_GO_DOWN, _("Next change"), "<Alt>Down", _("Go to the next change"), self.on_menu_edit_down_activate),
-            ("PrevChange", gtk.STOCK_GO_UP,   _("Previous change"), "<Alt>Up", _("Go to the previous change"), self.on_menu_edit_up_activate),
-            ("OpenExternal", None, _("Open externally"), None, _("Open selected file or directory in the default external application"), self.on_open_external),
+            ("NextChange", gtk.STOCK_GO_DOWN, _("Next Change"), "<Alt>Down",
+                _("Go to the next change"),
+                self.on_menu_edit_down_activate),
+            ("PrevChange", gtk.STOCK_GO_UP, _("Previous Change"), "<Alt>Up",
+                _("Go to the previous change"),
+                self.on_menu_edit_up_activate),
+            ("OpenExternal", None, _("Open Externally"), None,
+                _("Open selected file or directory in the default external "
+                  "application"),
+                self.on_open_external),
 
             ("ViewMenu", None, _("_View")),
-            ("FileStatus",  None, _("File status")),
-            ("VcStatus",    None, _("Version status")),
-            ("FileFilters",  None, _("File filters")),
-            ("Stop",    gtk.STOCK_STOP,     None, "Escape", _("Stop the current action"), self.on_toolbar_stop_clicked),
-            ("Refresh", gtk.STOCK_REFRESH,  None, "<control>R", _("Refresh the view"), self.on_menu_refresh_activate),
-            ("Reload",  gtk.STOCK_REFRESH,  _("Reload"), "<control><shift>R", _("Reload the comparison"), self.on_menu_reload_activate),
+            ("FileStatus", None, _("File Status")),
+            ("VcStatus", None, _("Version Status")),
+            ("FileFilters", None, _("File Filters")),
+            ("Stop", gtk.STOCK_STOP, None, "Escape",
+                _("Stop the current action"),
+                self.on_toolbar_stop_clicked),
+            ("Refresh", gtk.STOCK_REFRESH, None, "<control>R",
+                _("Refresh the view"),
+                self.on_menu_refresh_activate),
 
             ("TabMenu", None, _("_Tabs")),
-            ("PrevTab",   None, _("_Previous Tab"), "<Ctrl><Alt>Page_Up", _("Activate previous tab"), self.on_prev_tab),
-            ("NextTab",   None, _("_Next Tab"), "<Ctrl><Alt>Page_Down", _("Activate next tab"), self.on_next_tab),
-            ("MoveTabPrev", None, _("Move Tab _Left"), "<Ctrl><Alt><Shift>Page_Up", _("Move current tab to left"), self.on_move_tab_prev),
-            ("MoveTabNext", None, _("Move Tab _Right"), "<Ctrl><Alt><Shift>Page_Down", _("Move current tab to right"), self.on_move_tab_next),
+            ("PrevTab",   None, _("_Previous Tab"), "<Ctrl><Alt>Page_Up",
+                _("Activate previous tab"),
+                self.on_prev_tab),
+            ("NextTab",   None, _("_Next Tab"), "<Ctrl><Alt>Page_Down",
+                _("Activate next tab"),
+                self.on_next_tab),
+            ("MoveTabPrev", None,
+                _("Move Tab _Left"), "<Ctrl><Alt><Shift>Page_Up",
+                _("Move current tab to left"),
+                self.on_move_tab_prev),
+            ("MoveTabNext", None,
+                _("Move Tab _Right"), "<Ctrl><Alt><Shift>Page_Down",
+                _("Move current tab to right"),
+                self.on_move_tab_next),
 
             ("HelpMenu", None, _("_Help")),
-            ("Help",        gtk.STOCK_HELP,  _("_Contents"), "F1", _("Open the Meld manual"), self.on_menu_help_activate),
-            ("BugReport",   gtk.STOCK_DIALOG_WARNING, _("Report _Bug"), None, _("Report a bug in Meld"), self.on_menu_help_bug_activate),
-            ("About",       gtk.STOCK_ABOUT, None, None, _("About this program"), self.on_menu_about_activate),
+            ("Help", gtk.STOCK_HELP, _("_Contents"), "F1",
+                _("Open the Meld manual"), self.on_menu_help_activate),
+            ("BugReport", gtk.STOCK_DIALOG_WARNING, _("Report _Bug"), None,
+                _("Report a bug in Meld"),
+                self.on_menu_help_bug_activate),
+            ("About", gtk.STOCK_ABOUT, None, None,
+                _("About this program"),
+                self.on_menu_about_activate),
         )
         toggleactions = (
-            ("Fullscreen",       None, _("Full Screen"), "F11", _("View the comparison in full screen"), self.on_action_fullscreen_toggled, False),
-            ("ToolbarVisible",   None, _("_Toolbar"),   None, _("Show or hide the toolbar"),   self.on_menu_toolbar_toggled,   app.prefs.toolbar_visible),
-            ("StatusbarVisible", None, _("_Statusbar"), None, _("Show or hide the statusbar"), self.on_menu_statusbar_toggled, app.prefs.statusbar_visible)
+            ("Fullscreen", None, _("Fullscreen"), "F11",
+                _("View the comparison in fullscreen"),
+                self.on_action_fullscreen_toggled, False),
+            ("ToolbarVisible", None, _("_Toolbar"), None,
+                _("Show or hide the toolbar"),
+                self.on_menu_toolbar_toggled, app.prefs.toolbar_visible),
+            ("StatusbarVisible", None, _("_Statusbar"), None,
+                _("Show or hide the statusbar"),
+                self.on_menu_statusbar_toggled, app.prefs.statusbar_visible)
         )
         ui_file = paths.ui_dir("meldapp-ui.xml")
         self.actiongroup = gtk.ActionGroup('MainActions')
@@ -141,6 +198,8 @@ class MeldWindow(gnomeglade.Component):
         accels.connect_group(keyval, mask, 0, self.on_menu_edit_down_activate)
         (keyval, mask) = gtk.accelerator_parse("<Ctrl>E")
         accels.connect_group(keyval, mask, 0, self.on_menu_edit_up_activate)
+        (keyval, mask) = gtk.accelerator_parse("F5")
+        accels.connect_group(keyval, mask, 0, self.on_menu_refresh_activate)
 
         # Initialise sensitivity for important actions
         self.actiongroup.get_action("Stop").set_sensitive(False)
@@ -151,7 +210,7 @@ class MeldWindow(gnomeglade.Component):
         self._menu_context = self.statusbar.get_context_id("Tooltips")
         self.widget.drag_dest_set(
             gtk.DEST_DEFAULT_MOTION | gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_DROP,
-            [ ('text/uri-list', 0, 0) ],
+            [('text/uri-list', 0, 0)],
             gtk.gdk.ACTION_COPY)
         self.widget.connect("drag_data_received",
                             self.on_widget_drag_data_received)
@@ -161,7 +220,7 @@ class MeldWindow(gnomeglade.Component):
         app.prefs.notify_add(self.on_preference_changed)
         self.idle_hooked = 0
         self.scheduler = task.LifoScheduler()
-        self.scheduler.connect("runnable", self.on_scheduler_runnable )
+        self.scheduler.connect("runnable", self.on_scheduler_runnable)
         self.widget.set_default_size(app.prefs.window_size_x, app.prefs.window_size_y)
         self.ui.ensure_update()
         self.widget.show()
@@ -170,7 +229,7 @@ class MeldWindow(gnomeglade.Component):
         self.widget.connect('focus_in_event', self.on_focus_change)
         self.widget.connect('focus_out_event', self.on_focus_change)
 
-    def on_focus_change(self, widget, event, callback_data = None):
+    def on_focus_change(self, widget, event, callback_data=None):
         for idx in range(self.notebook.get_n_pages()):
             w = self.notebook.get_nth_page(idx)
             if hasattr(w.get_data("pyobject"), 'on_focus_change'):
@@ -262,7 +321,6 @@ class MeldWindow(gnomeglade.Component):
         have_focus = current_page != -1
         self.actiongroup.get_action("Close").set_sensitive(have_focus)
         self.actiongroup.get_action("Refresh").set_sensitive(have_focus)
-        self.actiongroup.get_action("Reload").set_sensitive(have_focus)
         if not have_focus:
             self.actiongroup.get_action("PrevChange").set_sensitive(False)
             self.actiongroup.get_action("NextChange").set_sensitive(False)
@@ -408,9 +466,6 @@ class MeldWindow(gnomeglade.Component):
     def on_menu_refresh_activate(self, *extra):
         self.current_doc().on_refresh_activate()
 
-    def on_menu_reload_activate(self, *extra):
-        self.current_doc().on_reload_activate()
-  
     def on_menu_find_activate(self, *extra):
         self.current_doc().on_find_activate()
 
@@ -536,6 +591,7 @@ class MeldWindow(gnomeglade.Component):
             if group is None:
                 group = action
             action.set_active(current_page == i)
+
             def current_tab_changed_cb(action, current):
                 if action == current:
                     self.notebook.set_current_page(action.get_current_value())
@@ -620,7 +676,7 @@ class MeldWindow(gnomeglade.Component):
         return doc
 
     def append_dirdiff(self, dirs, auto_compare=False):
-        assert len(dirs) in (1,2,3)
+        assert len(dirs) in (1, 2, 3)
         doc = dirdiff.DirDiff(app.prefs, len(dirs))
         self._append_page(doc, "folder")
         doc.set_locations(dirs)
@@ -663,11 +719,10 @@ class MeldWindow(gnomeglade.Component):
                     else:
                         # exit at first non found directory + file
                         misc.run_dialog(_("Cannot compare a mixture of files and directories.\n"),
-                                          parent=self,
-                                          buttonstype=gtk.BUTTONS_OK)
+                                        parent=self, buttonstype=gtk.BUTTONS_OK)
                         return
                 else:
-                     lastfilename = os.path.basename(elem)
+                    lastfilename = os.path.basename(elem)
                 builtfilelist.append(elem)
             return self.append_filediff(builtfilelist)
         elif dirslist:
@@ -698,11 +753,13 @@ class MeldWindow(gnomeglade.Component):
             tab = self.append_vcview(files[0])
         else:  # comparison_type == recent.TYPE_FILE:
             tab = self.append_filediff(files)
+        self.notebook.set_current_page(self.notebook.page_num(tab.widget))
         app.recent_comparisons.add(tab)
         return tab
 
     def _single_file_open(self, path):
         doc = vcview.VcView(app.prefs)
+
         def cleanup():
             self.scheduler.remove_scheduler(doc.scheduler)
         self.scheduler.add_task(cleanup)
@@ -737,6 +794,6 @@ class MeldWindow(gnomeglade.Component):
                 return page
 
         class DummyDoc(object):
-            def __getattr__(self, a): return lambda *x: None
+            def __getattr__(self, a):
+                return lambda *x: None
         return DummyDoc()
-
