@@ -21,6 +21,7 @@ import sys
 import gio
 import gtk
 import gobject
+import pango
 import atk
 # gconf is also imported; see end of HistoryEntry class for details
 from gettext import gettext as _
@@ -35,6 +36,7 @@ from ..util.compat import text_type
 MIN_ITEM_LEN = 3
 HISTORY_ENTRY_HISTORY_LENGTH_DEFAULT = 10
 
+
 def _remove_item(store, text):
     if text is None:
         return False
@@ -45,47 +47,38 @@ def _remove_item(store, text):
             return True
     return False
 
+
 def _clamp_list_store(liststore, max_items):
     try:
-        it = liststore.get_iter(max_items - 1) # -1 because TreePath counts from 0
+        # -1 because TreePath counts from 0
+        it = liststore.get_iter(max_items - 1)
     except ValueError:
         return
     valid = True
     while valid:
         valid = liststore.remove(it)
 
+
 def _escape_cell_data_func(col, renderer, model, it, escape_func):
     string = model.get(it, 0)
     escaped = escape_func(string)
     renderer.set("text", escaped)
 
 
-class HistoryEntry(gtk.ComboBoxEntry):
-    __gtype_name__ = "HistoryEntry"
-
-    __gproperties__ = {
-        "history-id":      (str, "History ID",
-                            "Identifier associated with entry's history store",
-                            None, gobject.PARAM_READWRITE),
-    }
+class HistoryWidget(object):
 
     def __init__(self, history_id=None, enable_completion=False, **kwargs):
-        super(HistoryEntry, self).__init__(**kwargs)
-
-        self.__history_id = history_id
-        self.__history_length = HISTORY_ENTRY_HISTORY_LENGTH_DEFAULT
-        self.__completion = None
+        self._history_id = history_id
+        self._history_length = HISTORY_ENTRY_HISTORY_LENGTH_DEFAULT
+        self._completion = None
         self._get_gconf_client()
 
         self.set_model(gtk.ListStore(str))
-        self.props.text_column = 0
-
         self.set_enable_completion(enable_completion)
 
-
     def do_get_property(self, pspec):
         if pspec.name == "history-id":
-            return self.__history_id
+            return self._history_id
         else:
             raise AttributeError("Unknown property: %s" % pspec.name)
 
@@ -95,36 +88,36 @@ class HistoryEntry(gtk.ComboBoxEntry):
             # things might happen
             store = self.get_model()
             store.clear()
-            self.__history_id = value
+            self._history_id = value
             self._load_history()
         else:
             raise AttributeError("Unknown property: %s" % pspec.name)
 
     def _get_gconf_client(self):
-        self.__gconf_client = gconf.client_get_default()
+        self._gconf_client = gconf.client_get_default()
 
-    def __get_history_key(self):
+    def _get_history_key(self):
         # We store data under /apps/gnome-settings/ like GnomeEntry did.
-        if not self.__history_id:
+        if not self._history_id:
             return None
-        key = ''.join(["/apps/gnome-settings/","meld","/history-",
-                          gconf.escape_key(self.__history_id, -1)])
+        key = ''.join(["/apps/gnome-settings/", "meld", "/history-",
+                       gconf.escape_key(self._history_id, -1)])
         return key
 
     def _save_history(self):
-        key = self.__get_history_key()
+        key = self._get_history_key()
         if key is None:
             return
         gconf_items = [row[0] for row in self.get_model()]
-        self.__gconf_client.set_list(key, gconf.VALUE_STRING, gconf_items)
+        self._gconf_client.set_list(key, gconf.VALUE_STRING, gconf_items)
 
-    def __insert_history_item(self, text, prepend):
+    def _insert_history_item(self, text, prepend):
         if len(text) <= MIN_ITEM_LEN:
             return
 
         store = self.get_model()
         if not _remove_item(store, text):
-            _clamp_list_store(store, self.__history_length - 1)
+            _clamp_list_store(store, self._history_length - 1)
 
         if (prepend):
             store.insert(0, (text,))
@@ -132,26 +125,26 @@ class HistoryEntry(gtk.ComboBoxEntry):
             store.append((text,))
         self._save_history()
 
-    def prepend_text(self, text):
+    def prepend_history(self, text):
         if not text:
             return
-        self.__insert_history_item(text, True)
+        self._insert_history_item(text, True)
 
-    def append_text(self, text):
+    def append_history(self, text):
         if not text:
             return
-        self.__insert_history_item(text, False)
+        self._insert_history_item(text, False)
 
     def _load_history(self):
-        key = self.__get_history_key()
+        key = self._get_history_key()
         if key is None:
             return
-        gconf_items = self.__gconf_client.get_list(key, gconf.VALUE_STRING)
+        gconf_items = self._gconf_client.get_list(key, gconf.VALUE_STRING)
 
         store = self.get_model()
         store.clear()
 
-        for item in gconf_items[:self.__history_length - 1]:
+        for item in gconf_items[:self._history_length - 1]:
             store.append((item,))
 
     def clear(self):
@@ -162,32 +155,32 @@ class HistoryEntry(gtk.ComboBoxEntry):
     def set_history_length(self, max_saved):
         if max_saved <= 0:
             return
-        self.__history_length = max_saved
+        self._history_length = max_saved
         if len(self.get_model()) > max_saved:
             self._load_history()
 
     def get_history_length(self):
-        return self.__history_length
+        return self._history_length
 
     def set_enable_completion(self, enable):
         if enable:
-            if self.__completion is not None:
+            if self._completion is not None:
                 return
-            self.__completion = gtk.EntryCompletion()
-            self.__completion.set_model(self.get_model())
-            self.__completion.set_text_column(0)
-            self.__completion.set_minimum_key_length(MIN_ITEM_LEN)
-            self.__completion.set_popup_completion(False)
-            self.__completion.set_inline_completion(True)
-            self.child.set_completion(self.__completion)
+            self._completion = gtk.EntryCompletion()
+            self._completion.set_model(self.get_model())
+            self._completion.set_text_column(0)
+            self._completion.set_minimum_key_length(MIN_ITEM_LEN)
+            self._completion.set_popup_completion(False)
+            self._completion.set_inline_completion(True)
+            self.child.set_completion(self._completion)
         else:
-            if self.__completion is None:
+            if self._completion is None:
                 return
             self.get_entry().set_completion(None)
-            self.__completion = None
+            self._completion = None
 
     def get_enable_completion(self):
-        return self.__completion is not None
+        return self._completion is not None
 
     def get_entry(self):
         return self.child
@@ -206,6 +199,91 @@ class HistoryEntry(gtk.ComboBoxEntry):
         else:
             self.set_cell_data_func(cells[0], None, None)
 
+
+class HistoryCombo(gtk.ComboBox, HistoryWidget):
+    __gtype_name__ = "HistoryCombo"
+
+    __gproperties__ = {
+        "history-id": (str, "History ID",
+                       "Identifier associated with entry's history store",
+                       None, gobject.PARAM_READWRITE),
+    }
+
+    def __init__(self, history_id=None, **kwargs):
+        super(HistoryCombo, self).__init__(**kwargs)
+        HistoryWidget.__init__(self, history_id)
+        self.set_model(gtk.ListStore(str, str))
+        rentext = gtk.CellRendererText()
+        rentext.props.width_chars = 60
+        rentext.props.ellipsize = pango.ELLIPSIZE_END
+        self.pack_start(rentext, True)
+        self.set_attributes(rentext, text=0)
+
+    def _save_history(self):
+        key = self._get_history_key()
+        if key is None:
+            return
+        gconf_items = [row[1] for row in self.get_model()]
+        self._gconf_client.set_list(key, gconf.VALUE_STRING, gconf_items)
+
+    def _insert_history_item(self, text, prepend):
+        if len(text) <= MIN_ITEM_LEN:
+            return
+
+        # Redefining here to key off the full text, not the first line
+        def _remove_item(store, text):
+            if text is None:
+                return False
+
+            for row in store:
+                if row[1] == text:
+                    store.remove(row.iter)
+                    return True
+            return False
+
+        store = self.get_model()
+        if not _remove_item(store, text):
+            _clamp_list_store(store, self._history_length - 1)
+
+        row = (text.splitlines()[0], text)
+
+        if (prepend):
+            store.insert(0, row)
+        else:
+            store.append(row)
+        self._save_history()
+
+    def _load_history(self):
+        key = self._get_history_key()
+        if key is None:
+            return
+        gconf_items = self._gconf_client.get_list(key, gconf.VALUE_STRING)
+
+        store = self.get_model()
+        store.clear()
+
+        # This override is here to handle multi-line commit messages, and is
+        # specific to HistoryCombo use in VcView.
+        for item in gconf_items[:self._history_length - 1]:
+            firstline = item.splitlines()[0]
+            store.append((firstline, item))
+
+
+class HistoryEntry(gtk.ComboBoxEntry, HistoryWidget):
+    __gtype_name__ = "HistoryEntry"
+
+    __gproperties__ = {
+        "history-id": (str, "History ID",
+                       "Identifier associated with entry's history store",
+                       None, gobject.PARAM_READWRITE),
+    }
+
+    def __init__(self, history_id=None, enable_completion=False, **kwargs):
+        super(HistoryEntry, self).__init__(**kwargs)
+        HistoryWidget.__init__(self, history_id, enable_completion)
+        self.props.text_column = 0
+
+
 try:
     import gconf
 except ImportError:
@@ -352,10 +430,10 @@ class HistoryFileEntry(gtk.HBox, gtk.Editable):
         last_open[self.props.history_id] = path
 
     def append_history(self, text):
-        self.__gentry.append_text(text)
+        self.__gentry.append_history(text)
 
     def prepend_history(self, text):
-        self.__gentry.prepend_text(text)
+        self.__gentry.prepend_history(text)
 
     def focus_entry(self):
         self.__gentry.focus_entry()
