@@ -37,12 +37,8 @@ from . import tree
 from . import vc
 from .ui import emblemcellrenderer
 from .ui import gnomeglade
+from meld.vc import _null
 
-################################################################################
-#
-# Local Functions
-#
-################################################################################
 
 def _commonprefix(files):
     if len(files) != 1:
@@ -58,8 +54,8 @@ def cleanup_temp():
     # not marked for translation.
     for f in _temp_files:
         try:
-            assert os.path.exists(f) and os.path.isabs(f) and \
-                   os.path.dirname(f) == temp_location
+            assert (os.path.exists(f) and os.path.isabs(f) and
+                    os.path.dirname(f) == temp_location)
             # Windows throws permissions errors if we remove read-only files
             if os.name == "nt":
                 os.chmod(f, stat.S_IWRITE)
@@ -70,8 +66,8 @@ def cleanup_temp():
                   file=sys.stderr)
     for f in _temp_dirs:
         try:
-            assert os.path.exists(f) and os.path.isabs(f) and \
-                   os.path.dirname(f) == temp_location
+            assert (os.path.exists(f) and os.path.isabs(f) and
+                    os.path.dirname(f) == temp_location)
             shutil.rmtree(f, ignore_errors=1)
         except:
             except_str = "{0[0]}: \"{0[1]}\"".format(sys.exc_info())
@@ -82,42 +78,119 @@ _temp_dirs, _temp_files = [], []
 atexit.register(cleanup_temp)
 
 
-################################################################################
-#
-# CommitDialog
-#
-################################################################################
 class CommitDialog(gnomeglade.Component):
+
     def __init__(self, parent):
-        gnomeglade.Component.__init__(self, paths.ui_dir("vcview.ui"), "commitdialog")
+        gnomeglade.Component.__init__(self, paths.ui_dir("vcview.ui"),
+                                      "commitdialog")
         self.parent = parent
-        self.widget.set_transient_for( parent.widget.get_toplevel() )
+        self.widget.set_transient_for(parent.widget.get_toplevel())
         selected = parent._get_selected_files()
-        topdir = _commonprefix(selected)
-        selected = [ s[len(topdir):] for s in selected ]
-        self.changedfiles.set_text( ("(in %s) "%topdir) + " ".join(selected) )
+
+        try:
+            to_commit = parent.vc.get_files_to_commit(selected)
+            topdir = parent.vc.root
+            if to_commit:
+                to_commit = ["\t" + s for s in to_commit]
+            else:
+                to_commit = ["\t" + _("No files will be committed")]
+        except NotImplementedError:
+            topdir = _commonprefix(selected)
+            to_commit = ["\t" + s[len(topdir) + 1:] for s in selected]
+        self.changedfiles.set_text("(in %s)\n%s" %
+                                   (topdir, "\n".join(to_commit)))
+
+        fontdesc = pango.FontDescription(self.parent.prefs.get_current_font())
+        self.textview.modify_font(fontdesc)
+        commit_prefill = self.parent.vc.get_commit_message_prefill()
+        if commit_prefill:
+            buf = self.textview.get_buffer()
+            buf.set_text(commit_prefill)
+            buf.place_cursor(buf.get_start_iter())
+
+        # Try and make the textview wide enough for a standard 80-character
+        # commit message.
+        context = self.textview.get_pango_context()
+        metrics = context.get_metrics(fontdesc, context.get_language())
+        char_width = metrics.get_approximate_char_width()
+        self.textview.set_size_request(80 * pango.PIXELS(char_width), -1)
+
         self.widget.show_all()
 
     def run(self):
-        self.previousentry.child.set_editable(False)
-        self.previousentry.set_active(0)
+        self.previousentry.set_active(-1)
         self.textview.grab_focus()
-        buf = self.textview.get_buffer()
-        buf.place_cursor( buf.get_start_iter() )
-        buf.move_mark( buf.get_selection_bound(), buf.get_end_iter() )
         response = self.widget.run()
-        msg = buf.get_text(buf.get_start_iter(), buf.get_end_iter(), 0)
         if response == gtk.RESPONSE_OK:
-            self.parent._command_on_selected( self.parent.vc.commit_command(msg) )
-        if len(msg.strip()):
-            self.previousentry.prepend_text(msg)
+            buf = self.textview.get_buffer()
+            msg = buf.get_text(*buf.get_bounds(), include_hidden_chars=False)
+            self.parent._command_on_selected(
+                self.parent.vc.commit_command(msg))
+            if msg.strip():
+                self.previousentry.prepend_history(msg)
         self.widget.destroy()
-    def on_previousentry_activate(self, gentry):
-        buf = self.textview.get_buffer()
-        buf.set_text( gentry.child.get_text() )
 
-COL_LOCATION, COL_STATUS, COL_REVISION, COL_TAG, COL_OPTIONS, COL_END = \
-    list(range(tree.COL_END, tree.COL_END+6))
+    def on_previousentry_activate(self, gentry):
+        idx = gentry.get_active()
+        if idx != -1:
+            model = gentry.get_model()
+            buf = self.textview.get_buffer()
+            buf.set_text(model[idx][1])
+
+
+class PushDialog(gnomeglade.Component):
+
+    def __init__(self, parent):
+        gnomeglade.Component.__init__(self, paths.ui_dir("vcview.ui"),
+                                      "pushdialog")
+        self.parent = parent
+        self.widget.set_transient_for(parent.widget.get_toplevel())
+        self.widget.show_all()
+
+    def run(self):
+        # TODO: Ask the VC for a more informative label for what will happen.
+        # In git, this is probably the parsed output of push --dry-run.
+
+        response = self.widget.run()
+        if response == gtk.RESPONSE_OK:
+            self.parent.vc.push(self.parent._command)
+        self.widget.destroy()
+
+
+class ConsoleStream(object):
+
+    def __init__(self, textview):
+        self.textview = textview
+        buf = textview.get_buffer()
+        self.command_tag = buf.create_tag("command")
+        self.command_tag.props.weight = pango.WEIGHT_BOLD
+        self.output_tag = buf.create_tag("output")
+        self.error_tag = buf.create_tag("error")
+        # FIXME: Need to add this to the gtkrc?
+        self.error_tag.props.foreground = "#cc0000"
+        self.end_mark = buf.create_mark(None, buf.get_end_iter(),
+                                        left_gravity=False)
+
+    def command(self, message):
+        self.write(message, self.command_tag)
+
+    def output(self, message):
+        self.write(message, self.output_tag)
+
+    def error(self, message):
+        self.write(message, self.error_tag)
+
+    def write(self, message, tag):
+        if not message:
+            return
+        buf = self.textview.get_buffer()
+        buf.insert_with_tags(buf.get_end_iter(), message, tag)
+        self.textview.scroll_mark_onscreen(self.end_mark)
+
+
+COL_LOCATION, COL_STATUS, COL_REVISION, COL_OPTIONS, COL_END = \
+    list(range(tree.COL_END, tree.COL_END + 5))
+
 
 class VcTreeStore(tree.DiffTreeStore):
     def __init__(self):
@@ -139,14 +212,15 @@ entry_ignored  = lambda x: (x.state == tree.STATE_IGNORED) or x.isdir
 class VcView(melddoc.MeldDoc, gnomeglade.Component):
     # Map action names to VC commands and required arguments list
     action_vc_cmds_map = {
-                         "VcCompare": ("diff_command", ()),
-                         "VcCommit": ("commit_command", ("",)),
-                         "VcUpdate": ("update_command", ()),
-                         "VcAdd": ("add_command", ()),
-                         "VcResolved": ("resolved_command", ()),
-                         "VcRemove": ("remove_command", ()),
-                         "VcRevert": ("revert_command", ()),
-                         }
+        "VcCompare": ("diff_command", ()),
+        "VcCommit": ("commit_command", ("",)),
+        "VcUpdate": ("update_command", ()),
+        "VcPush": ("push", (lambda *args, **kwargs: None, )),
+        "VcAdd": ("add_command", ()),
+        "VcResolved": ("resolved_command", ()),
+        "VcRemove": ("remove_command", ()),
+        "VcRevert": ("revert_command", ()),
+    }
 
     state_actions = {
         "flatten": ("VcFlatten", None),
@@ -158,25 +232,55 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
 
     def __init__(self, prefs):
         melddoc.MeldDoc.__init__(self, prefs)
-        gnomeglade.Component.__init__(self, paths.ui_dir("vcview.ui"), "vcview")
+        gnomeglade.Component.__init__(self, paths.ui_dir("vcview.ui"),
+                                      "vcview")
 
         actions = (
-            ("VcCompare",       gtk.STOCK_DIALOG_INFO,      _("_Compare"),      None, _("Compare selected"), self.on_button_diff_clicked),
-            ("VcCommit",        "vc-commit-24",             _("Co_mmit"),       None, _("Commit"), self.on_button_commit_clicked),
-            ("VcUpdate",        "vc-update-24",             _("_Update"),       None, _("Update"), self.on_button_update_clicked),
-            ("VcAdd",           "vc-add-24",                _("_Add"),          None, _("Add to VC"), self.on_button_add_clicked),
-            ("VcRemove",        "vc-remove-24",             _("_Remove"),       None, _("Remove from VC"), self.on_button_remove_clicked),
-            ("VcResolved",      "vc-resolve-24",            _("_Resolved"),     None, _("Mark as resolved for VC"), self.on_button_resolved_clicked),
-            ("VcRevert",        gtk.STOCK_REVERT_TO_SAVED,  None,               None, _("Revert to original"), self.on_button_revert_clicked),
-            ("VcDeleteLocally", gtk.STOCK_DELETE,           None,               None, _("Delete locally"), self.on_button_delete_clicked),
+            ("VcCompare", gtk.STOCK_DIALOG_INFO, _("_Compare"), None,
+                _("Compare selected files"),
+                self.on_button_diff_clicked),
+            ("VcCommit", "vc-commit-24", _("Co_mmit..."), None,
+                _("Commit changes to version control"),
+                self.on_button_commit_clicked),
+            ("VcUpdate", "vc-update-24", _("_Update"), None,
+                _("Update working copy from version control"),
+                self.on_button_update_clicked),
+            ("VcPush", "vc-push-24", _("_Push"), None,
+                _("Push local changes to remote"),
+                self.on_button_push_clicked),
+            ("VcAdd", "vc-add-24", _("_Add"), None,
+                _("Add to version control"),
+                self.on_button_add_clicked),
+            ("VcRemove", "vc-remove-24", _("_Remove"), None,
+                _("Remove from version control"),
+                self.on_button_remove_clicked),
+            ("VcResolved", "vc-resolve-24", _("Mar_k as Resolved"), None,
+                _("Mark as resolved in version control"),
+                self.on_button_resolved_clicked),
+            ("VcRevert", gtk.STOCK_REVERT_TO_SAVED, _("Re_vert"), None,
+                _("Revert working copy to original state"),
+                self.on_button_revert_clicked),
+            ("VcDeleteLocally", gtk.STOCK_DELETE, None, None,
+                _("Delete from working copy"),
+                self.on_button_delete_clicked),
         )
 
         toggleactions = (
-            ("VcFlatten",     gtk.STOCK_GOTO_BOTTOM, _("_Flatten"),  None, _("Flatten directories"), self.on_button_flatten_toggled, False),
-            ("VcShowModified","filter-modified-24",  _("_Modified"), None, _("Show modified"), self.on_filter_state_toggled, False),
-            ("VcShowNormal",  "filter-normal-24",    _("_Normal"),   None, _("Show normal"), self.on_filter_state_toggled, False),
-            ("VcShowNonVC",   "filter-nonvc-24",     _("Non _VC"),   None, _("Show unversioned files"), self.on_filter_state_toggled, False),
-            ("VcShowIgnored", "filter-ignored-24",   _("Ignored"),   None, _("Show ignored files"), self.on_filter_state_toggled, False),
+            ("VcFlatten", gtk.STOCK_GOTO_BOTTOM, _("_Flatten"),  None,
+                _("Flatten directories"),
+                self.on_button_flatten_toggled, False),
+            ("VcShowModified", "filter-modified-24", _("_Modified"), None,
+                _("Show modified files"),
+                self.on_filter_state_toggled, False),
+            ("VcShowNormal", "filter-normal-24", _("_Normal"), None,
+                _("Show normal files"),
+                self.on_filter_state_toggled, False),
+            ("VcShowNonVC", "filter-nonvc-24", _("Un_versioned"), None,
+                _("Show unversioned files"),
+                self.on_filter_state_toggled, False),
+            ("VcShowIgnored", "filter-ignored-24", _("Ignored"), None,
+                _("Show ignored files"),
+                self.on_filter_state_toggled, False),
         )
 
         self.ui_file = paths.ui_dir("vcview-ui.xml")
@@ -187,7 +291,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         for action in ("VcCompare", "VcFlatten", "VcShowModified",
                        "VcShowNormal", "VcShowNonVC", "VcShowIgnored"):
             self.actiongroup.get_action(action).props.is_important = True
-        for action in ("VcCommit", "VcUpdate", "VcAdd", "VcRemove",
+        for action in ("VcCommit", "VcUpdate", "VcPush", "VcAdd", "VcRemove",
                        "VcShowModified", "VcShowNormal", "VcShowNonVC",
                        "VcShowIgnored", "VcResolved"):
             button = self.actiongroup.get_action(action)
@@ -201,7 +305,10 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         self.treeview.set_headers_visible(1)
         self.treeview.set_search_equal_func(self.treeview_search_cb)
         self.current_path, self.prev_path, self.next_path = None, None, None
-        column = gtk.TreeViewColumn( _("Name") )
+
+        self.column_name_map = {}
+        column = gtk.TreeViewColumn(_("Name"))
+        column.set_resizable(True)
         renicon = emblemcellrenderer.EmblemCellRenderer()
         rentext = gtk.CellRendererText()
         column.pack_start(renicon, expand=0)
@@ -216,23 +323,25 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                     style=col_index(tree.COL_STYLE, 0),
                     weight=col_index(tree.COL_WEIGHT, 0),
                     strikethrough=col_index(tree.COL_STRIKE, 0))
+        column_index = self.treeview.append_column(column) - 1
+        self.column_name_map[vc.DATA_NAME] = column_index
 
-        self.treeview.append_column(column)
-
-        def addCol(name, num):
+        def addCol(name, num, data_name=None):
             column = gtk.TreeViewColumn(name)
+            column.set_resizable(True)
             rentext = gtk.CellRendererText()
             column.pack_start(rentext, expand=0)
-            column.set_attributes(rentext, markup=self.model.column_index(num, 0))
-            self.treeview.append_column(column)
+            column.set_attributes(rentext,
+                                  markup=self.model.column_index(num, 0))
+            column_index = self.treeview.append_column(column) - 1
+            if data_name:
+                self.column_name_map[data_name] = column_index
             return column
 
-        self.treeview_column_location = addCol( _("Location"), COL_LOCATION)
-        addCol(_("Status"), COL_STATUS)
-        addCol(_("Rev"), COL_REVISION)
-        addCol(_("Tag"), COL_TAG)
-        addCol(_("Options"), COL_OPTIONS)
-
+        self.treeview_column_location = addCol(_("Location"), COL_LOCATION)
+        addCol(_("Status"), COL_STATUS, vc.DATA_STATE)
+        addCol(_("Revision"), COL_REVISION, vc.DATA_REVISION)
+        addCol(_("Options"), COL_OPTIONS, vc.DATA_OPTIONS)
 
         self.state_filters = []
         for s in self.state_actions:
@@ -241,16 +350,6 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                 self.state_filters.append(s)
                 self.actiongroup.get_action(action_name).set_active(True)
 
-        class ConsoleStream(object):
-            def __init__(self, textview):
-                self.textview = textview
-                b = textview.get_buffer()
-                self.mark = b.create_mark("END", b.get_end_iter(), 0)
-            def write(self, s):
-                if s:
-                    b = self.textview.get_buffer()
-                    b.insert(b.get_end_iter(), s)
-                    self.textview.scroll_mark_onscreen( self.mark )
         self.consolestream = ConsoleStream(self.consoleview)
         self.location = None
         self.treeview_column_location.set_visible(self.actiongroup.get_action("VcFlatten").get_active())
@@ -275,6 +374,11 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         melddoc.MeldDoc.on_container_switch_in_event(self, ui)
         self.scheduler.add_task(self.on_treeview_cursor_changed)
 
+    def update_visible_columns(self):
+        for data_id in self.column_name_map:
+            col = self.treeview.get_column(self.column_name_map[data_id])
+            col.set_visible(data_id in self.vc.VC_COLUMNS)
+
     def update_actions_sensitivity(self):
         """Disable actions that use not implemented VC plugin methods
         """
@@ -293,8 +397,6 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         """Display VC plugin(s) that can handle the location"""
         self.combobox_vcs.lock = True
         self.combobox_vcs.get_model().clear()
-        tooltip_texts = [_("Choose one Version Control"),
-                         _("Only one Version Control in this directory")]
         default_active = -1
         valid_vcs = []
         # Try to keep the same VC plugin active on refresh()
@@ -315,36 +417,56 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             if not vc_installed(avc.CMD):
                 # TRANSLATORS: this is an error message when a version control
                 # application isn't installed or can't be found
-                err_str = _("%s Not Installed" % avc.CMD)
+                err_str = _("%s not installed" % avc.CMD)
             elif not avc.valid_repo():
                 # TRANSLATORS: this is an error message when a version
                 # controlled repository is invalid or corrupted
-                err_str = _("Invalid Repository")
+                err_str = _("Invalid repository")
             else:
                 valid_vcs.append(idx)
                 if (self.vc is not None and
-                     self.vc.__class__ == avc.__class__):
-                     default_active = idx
+                        self.vc.__class__ == avc.__class__):
+                    default_active = idx
 
             if err_str:
-                self.combobox_vcs.get_model().append( \
-                        [_("%s (%s)") % (avc.NAME, err_str), avc, False])
+                self.combobox_vcs.get_model().append(
+                    [_("%s (%s)") % (avc.NAME, err_str), avc, False])
             else:
                 self.combobox_vcs.get_model().append([avc.NAME, avc, True])
 
-        if valid_vcs and default_active == -1:
-            default_active = min(valid_vcs)
+        if not valid_vcs:
+            # If we didn't get any valid vcs then fallback to null
+            null_vcs = _null.Vc(vcs[0].location)
+            vcs.append(null_vcs)
+            self.combobox_vcs.get_model().insert(
+                0, [null_vcs.NAME, null_vcs, True])
+            default_active = 0
 
-        self.combobox_vcs.set_tooltip_text(tooltip_texts[len(vcs) == 1])
+        if default_active == -1:
+            if valid_vcs:
+                default_active = min(valid_vcs)
+            else:
+                default_active = 0
+
+        # If we only have the null VC, give a better error message.
+        if (len(vcs) == 1 and vcs[0].CMD == "true") or (len(valid_vcs) == 0):
+            tooltip = _("No valid version control system found in this folder")
+        elif len(vcs) == 1:
+            tooltip = _("Only one version control system found in this folder")
+        else:
+            tooltip = _("Choose which version control system to use")
+
+        self.combobox_vcs.set_tooltip_text(tooltip)
         self.combobox_vcs.set_sensitive(len(vcs) > 1)
         self.combobox_vcs.lock = False
         self.combobox_vcs.set_active(default_active)
-  
+
     def on_vc_change(self, cb):
         if not cb.lock:
             self.vc = cb.get_model()[cb.get_active_iter()][1]
             self._set_location(self.vc.location)
             self.update_actions_sensitivity()
+            self.update_visible_columns()
 
     def set_location(self, location):
         self.choose_vc(vc.get_vcs(os.path.abspath(location or ".")))
@@ -366,7 +488,16 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         # need to scan the rest of the repository
         if os.path.isdir(self.vc.location):
             root = self.model.get_iter_root()
+
+            try:
+                col = self.model.column_index(COL_OPTIONS, 0)
+                self.model.set_value(root, col,
+                                     self.vc.get_commits_to_push_summary())
+            except AttributeError:
+                pass
+
             self.scheduler.add_task(self._search_recursively_iter(root))
+            self.scheduler.add_task(self.on_treeview_selection_changed)
             self.scheduler.add_task(self.on_treeview_cursor_changed)
 
     def get_comparison(self):
@@ -458,11 +589,6 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                 yield 1
             prefix, patch = diff[0], diff[1]
 
-            try:
-                patch = self.vc.clean_patch(patch)
-            except AttributeError:
-                pass
-
             yield _("[%s] Applying patch") % self.label_text
             if patch:
                 applied = self.show_patch(prefix, patch, silent=silent_error)
@@ -482,8 +608,9 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                     continue
 
                 kwargs = {}
-                status = self.vc._get_tree_cache(path).get(path, None)
-                if status == tree.STATE_CONFLICT:
+                vc_file = self.vc.lookup_files(
+                    [], [(os.path.basename(path), path)])[1][0]
+                if vc_file.state == tree.STATE_CONFLICT:
                     # We use auto merge, so we create a new temp file
                     # for other, base and this, then set the output to
                     # the current file.
@@ -498,7 +625,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                             os.chmod(conflict_path, 0o444)
                             _temp_files.append(conflict_path)
                     # create-diff expects only the paths
-                    diffs = [path for path, is_temp in diffs]
+                    diffs = [p for p, is_temp in diffs]
 
                     # If we want to use auto-merge or use the merged
                     # output given by the VCS
@@ -553,11 +680,36 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         self.prefs.vc_status_filters = active_filters
         self.refresh()
 
-    def on_treeview_selection_changed(self, selection):
+    def on_treeview_selection_changed(self, selection=None):
+
+        def set_sensitive(action, sensitive):
+            self.actiongroup.get_action(action).set_sensitive(sensitive)
+
+        if selection is None:
+            selection = self.treeview.get_selection()
         model, rows = selection.get_selected_rows()
-        have_selection = bool(rows)
-        for action in self.valid_vc_actions:
-            self.actiongroup.get_action(action).set_sensitive(have_selection)
+        if hasattr(self.vc, 'update_actions_for_paths'):
+            paths = [self.model.value_path(model.get_iter(r), 0) for r in rows]
+            states = [self.model.get_state(model.get_iter(r), 0) for r in rows]
+            action_sensitivity = {
+                "VcCompare": False,
+                "VcCommit": False,
+                "VcUpdate": False,
+                "VcPush": False,
+                "VcAdd": False,
+                "VcResolved": False,
+                "VcRemove": False,
+                "VcRevert": False,
+                "VcDeleteLocally": bool(paths) and self.vc.root not in paths,
+            }
+            path_states = dict(zip(paths, states))
+            self.vc.update_actions_for_paths(path_states, action_sensitivity)
+            for action, sensitivity in action_sensitivity.items():
+                set_sensitive(action, sensitivity)
+        else:
+            have_selection = bool(rows)
+            for action in self.valid_vc_actions:
+                set_sensitive(action, have_selection)
 
     def _get_selected_files(self):
         model, rows = self.treeview.get_selection().get_selected_rows()
@@ -584,17 +736,18 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             workdir = self.vc.get_working_directory( _commonprefix(files) )
         files = [ relpath(workdir, f) for f in files ]
         r = None
-        self.consolestream.write( misc.shelljoin(command+files) + " (in %s)\n" % workdir)
+        self.consolestream.command(misc.shelljoin(command + files) + " (in %s)\n" % workdir)
         readiter = misc.read_pipe_iter(command + files, self.consolestream,
                                        workdir=workdir)
         try:
             while r is None:
                 r = next(readiter)
-                self.consolestream.write(r)
+                self.consolestream.output(r)
                 yield 1
         except IOError as e:
             misc.run_dialog("Error running command.\n'%s'\n\nThe error was:\n%s" % ( misc.shelljoin(command), e),
                 parent=self, messagetype=gtk.MESSAGE_ERROR)
+        self.consolestream.output("\n")
         if refresh:
             self.refresh_partial(workdir)
         yield workdir, r
@@ -611,7 +764,13 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             self._command(command, files, refresh)
 
     def on_button_update_clicked(self, obj):
-        self._command_on_selected(self.vc.update_command())
+        try:
+            self.vc.update(self._command, self._get_selected_files())
+        except NotImplementedError:
+            self._command_on_selected(self.vc.update_command())
+
+    def on_button_push_clicked(self, obj):
+        PushDialog(self).run()
 
     def on_button_commit_clicked(self, obj):
         CommitDialog(self).run()
@@ -638,7 +797,10 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         self._command_on_selected(self.vc.resolved_command())
 
     def on_button_revert_clicked(self, obj):
-        self._command_on_selected(self.vc.revert_command())
+        try:
+            self.vc.revert(self._command, self._get_selected_files())
+        except NotImplementedError:
+            self._command_on_selected(self.vc.revert_command())
 
     def on_button_delete_clicked(self, obj):
         files = self._get_selected_files()
@@ -742,7 +904,11 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                 self.model.set_value(newiter, self.model.column_index( tree.COL_PATH, 0), where)
                 self.model.set_path_state(newiter, 0, tree.STATE_NORMAL, True)
                 self.model.remove(it)
+                self.treeview.grab_focus()
+                self.treeview.get_selection().select_iter(newiter)
                 self.scheduler.add_task(self._search_recursively_iter(newiter))
+                self.scheduler.add_task(self.on_treeview_selection_changed)
+                self.scheduler.add_task(self.on_treeview_cursor_changed)
         else: # XXX fixme
             self.refresh()
 
@@ -754,7 +920,6 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         setcol(COL_LOCATION, location)
         setcol(COL_STATUS, e.get_status())
         setcol(COL_REVISION, e.rev)
-        setcol(COL_TAG, e.tag)
         setcol(COL_OPTIONS, e.options)
 
     def on_file_changed(self, filename):
@@ -860,7 +1025,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             self.treeview.expand_to_path(path)
             self.treeview.set_cursor(path)
 
-    def on_reload_activate(self, *extra):
+    def on_refresh_activate(self, *extra):
         self.on_fileentry_activate(self.fileentry)
 
     def on_find_activate(self, *extra):
